﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using SabberStoneCore.Enums;
using SabberStoneCore.Model;
using SabberStoneCore.Model.Entities;
using SabberStoneCore.Tasks;
using SabberStoneCoreAi.Agent;
using SabberStoneCoreAi.Nodes;
using SabberStoneCoreAi.POGame;
using SabberStoneCoreAi.Score;
using SabberStoneCore.Tasks.PlayerTasks;


//Developed by Lars Wagner
namespace SabberStoneCoreAi.Agent
{
	class BotLarsWagnerUpdated : AbstractAgent
	{
		private Random Rnd = new Random();
		private int countFlamestrike;
		List<PlayHistoryEntry> oppPlayedCards = new List<PlayHistoryEntry>();

		bool methodSwitch = false;

		public BotLarsWagnerUpdated(bool option)
		{
			methodSwitch = option;
		}

		public BotLarsWagnerUpdated() : this(true){
		}

		public override void FinalizeAgent()
		{
		}

		public override void FinalizeGame()
		{
			countFlamestrike = 0;
		}

		public override PlayerTask GetMove(SabberStoneCoreAi.POGame.POGame poGame)
		{
			List<int> values = new List<int>();
			List<PlayerTask> turnList = new List<PlayerTask>();
			
			PlayerTask bestOption = null;
			int bestValue = int.MinValue;

			Score.ControlScore controlScore = new ControlScore();
			Score.AggroScore aggroScore = new AggroScore();
			Score.MidRangeScore midrange = new MidRangeScore();

			Controller control = poGame.CurrentPlayer;

			Dictionary<PlayerTask, POGame.POGame> simulated = poGame.Simulate(control.Options());


			if (control.Options().Count == 1)
				return control.Options()[0];

			oppPlayedCards = poGame.CurrentOpponent.PlayHistory;

			foreach (PlayHistoryEntry h in oppPlayedCards)
			{
				if (h.SourceCard.Name == "Flamestrike")
				{
					countFlamestrike = 1;
				}
					
			}

			foreach (PlayerTask k in simulated.Keys)
			{

				if (k.PlayerTaskType == PlayerTaskType.END_TURN)
					continue;

				if(methodSwitch)
				{
					if (poGame.CurrentPlayer.BoardZone.Count >= 4 && k.PlayerTaskType == PlayerTaskType.PLAY_CARD && k.Source.Card.Type == CardType.MINION)
						continue;
				}
				else
				{
					if (poGame.CurrentPlayer.BoardZone.Count >= 3 && k.PlayerTaskType == PlayerTaskType.PLAY_CARD && k.Source.Card.Type == CardType.MINION)
						continue;
				}
				

				if (poGame.CurrentOpponent.HeroClass == CardClass.MAGE && poGame.CurrentOpponent.BaseMana >= 7 &&
				    countFlamestrike == 0 && k.PlayerTaskType == PlayerTaskType.PLAY_CARD && k.Source.Card.Type == CardType.MINION && k.Source.Card[GameTag.HEALTH] <= 4 && poGame.CurrentPlayer.BoardZone.Count >= 2)
				{
					continue;
				}

				//controller of simulated option
				control = simulated.First(x => x.Key == k).Value?.CurrentPlayer;

				if(control == null)
					continue;

				//set controller on rating function
				//controlScore.Controller = control;
				midrange.Controller = control;

				//rate current option
				var currentValue = midrange.Rate();

				if (bestValue < currentValue)
				{
					bestValue = currentValue;
					bestOption = k;
				}
			}

			//debug(turnList, values, bestOption, bestValue, poGame);

			return bestOption ??
			       (bestOption = poGame.CurrentPlayer.Options().Find(x => x.PlayerTaskType == PlayerTaskType.END_TURN));
		}

		public override void InitializeAgent()
		{
			Rnd = new Random();
		}

		public override void InitializeGame()
		{
			countFlamestrike = 0;
		}

		public void debug(List<PlayerTask> options, List<int> values, PlayerTask bestOption, int bestValue, POGame.POGame poGame)
		{
			foreach (PlayerTask task in options)
			{
				Console.WriteLine("Current Turn Options: " + task.FullPrint());
			}

			Console.WriteLine("\n\n");

			foreach (int value in values)
			{
				Console.WriteLine("Current Turn Options values: " + value);
			}

			Console.WriteLine("\n\n");

			Console.WriteLine("Option: " + bestOption.FullPrint() + "     Value: " + bestValue);

			Console.WriteLine("My health: " + poGame.CurrentPlayer.Hero.Health);
			Console.WriteLine("Opponent health: " + poGame.CurrentOpponent.Hero.Health);
		}
	}
}
